@@ -6,11 +6,11 @@ import java.io.IOException;
  * Signals that a HTTP request resulted in a not OK HTTP response.
  */
 public class HttpStatusException extends IOException {
-    private int statusCode;
-    private String url;
+    private final int statusCode;
+    private final String url;
 
     public HttpStatusException(String message, int statusCode, String url) {
-        super(message);
+        super(message + ". Status=" + statusCode + ", URL=[" + url + "]");
         this.statusCode = statusCode;
         this.url = url;
     }
@@ -22,9 +22,4 @@ public class HttpStatusException extends IOException {
     public String getUrl() {
         return url;
     }
-
-    @Override
-    public String toString() {
-        return super.toString() + ". Status=" + statusCode + ", URL=" + url;
-    }
 }
