@@ -81,14 +81,16 @@ public final class ConstrainableInputStream extends BufferedInputStream {
         final ByteArrayOutputStream outStream = new ByteArrayOutputStream(bufferSize);
 
         int read;
+        int remaining = max;
         while (true) {
-            read = read(readBuffer, 0, bufferSize);
+            read = read(readBuffer, 0, localCapped ? Math.min(remaining, bufferSize) : bufferSize);
             if (read == -1) break;
             if (localCapped) { // this local byteBuffer cap may be smaller than the overall maxSize (like when reading first bytes)
-                if (read >= max) {
-                    outStream.write(readBuffer, 0, max);
+                if (read >= remaining) {
+                    outStream.write(readBuffer, 0, remaining);
                     break;
                 }
+                remaining -= read;
             }
             outStream.write(readBuffer, 0, read);
         }
