@@ -31,6 +31,7 @@ enum HtmlTreeBuilderState {
                     tb.settings.normalizeTag(d.getName()), d.getPublicIdentifier(), d.getSystemIdentifier());
                 doctype.setPubSysKey(d.getPubSysKey());
                 tb.getDocument().appendChild(doctype);
+                tb.onNodeInserted(doctype, t);
                 if (d.isForceQuirks())
                     tb.getDocument().quirksMode(Document.QuirksMode.quirks);
                 tb.transition(BeforeHtml);
@@ -384,8 +385,9 @@ enum HtmlTreeBuilderState {
                         return false; // ignore
                     } else {
                         tb.framesetOk(false);
-                        Element body = stack.get(1);
-                        if (startTag.hasAttributes()) {
+                        // will be on stack if this is a nested body. won't be if closed (which is a variance from spec, which leaves it on)
+                        Element body;
+                        if (startTag.hasAttributes() && (body = tb.getFromStack("body")) != null) { // we only ever put one body on stack
                             for (Attribute attribute : startTag.attributes) {
                                 if (!body.hasAttr(attribute.getKey()))
                                     body.attributes().put(attribute);
@@ -630,14 +632,38 @@ enum HtmlTreeBuilderState {
                     }
                     // todo - is this right? drops rp, rt if ruby not in scope?
                     break;
+                // InBodyStartEmptyFormatters:
+                case "area":
+                case "br":
+                case "embed":
+                case "img":
+                case "keygen":
+                case "wbr":
+                    tb.reconstructFormattingElements();
+                    tb.insertEmpty(startTag);
+                    tb.framesetOk(false);
+                    break;
+                // Formatters:
+                case "b":
+                case "big":
+                case "code":
+                case "em":
+                case "font":
+                case "i":
+                case "s":
+                case "small":
+                case "strike":
+                case "strong":
+                case "tt":
+                case "u":
+                    tb.reconstructFormattingElements();
+                    el = tb.insert(startTag);
+                    tb.pushActiveFormattingElements(el);
+                    break;
                 default:
                     // todo - bring scan groups in if desired
                     if (!Tag.isKnownTag(name)) { // no special rules for custom tags
                         tb.insert(startTag);
-                    } else if (inSorted(name, Constants.InBodyStartEmptyFormatters)) {
-                        tb.reconstructFormattingElements();
-                        tb.insertEmpty(startTag);
-                        tb.framesetOk(false);
                     } else if (inSorted(name, Constants.InBodyStartPClosers)) {
                         if (tb.inButtonScope("p")) {
                             tb.processEndTag("p");
@@ -645,10 +671,6 @@ enum HtmlTreeBuilderState {
                         tb.insert(startTag);
                     } else if (inSorted(name, Constants.InBodyStartToHead)) {
                         return tb.process(t, InHead);
-                    } else if (inSorted(name, Constants.Formatters)) {
-                        tb.reconstructFormattingElements();
-                        el = tb.insert(startTag);
-                        tb.pushActiveFormattingElements(el);
                     } else if (inSorted(name, Constants.InBodyStartApplets)) {
                         tb.reconstructFormattingElements();
                         tb.insert(startTag);
@@ -697,6 +719,7 @@ enum HtmlTreeBuilderState {
                         return false;
                     } else {
                         // todo: error if stack contains something not dd, dt, li, optgroup, option, p, rp, rt, tbody, td, tfoot, th, thead, tr, body, html
+                        anyOtherEndTag(t, tb);
                         tb.transition(AfterBody);
                     }
                     break;
@@ -997,8 +1020,7 @@ enum HtmlTreeBuilderState {
                         return false;
                     } else {
                         tb.popStackToClose(name);
-                        tb.resetInsertionMode();
-                        if (tb.state() == InTable) {
+                        if (!tb.resetInsertionMode()) {
                             // not per spec - but haven't transitioned out of table. so try something else
                             tb.insert(startTag);
                             return true;
@@ -1577,13 +1599,14 @@ enum HtmlTreeBuilderState {
                     tb.error(this);
                     return false;
                 } else {
+                    if (tb.onStack("html")) tb.popStackToClose("html");
                     tb.transition(AfterAfterBody);
                 }
             } else if (t.isEOF()) {
                 // chillax! we're done
             } else {
                 tb.error(this);
-                tb.transition(InBody);
+                tb.resetBody();
                 return tb.process(t);
             }
             return true;
@@ -1668,21 +1691,12 @@ enum HtmlTreeBuilderState {
             } else if (t.isDoctype() || (t.isStartTag() && t.asStartTag().normalName().equals("html"))) {
                 return tb.process(t, InBody);
             } else if (isWhitespace(t)) {
-                // allows space after </html>, and put the body back on stack to allow subsequent tags if any
-                // todo - might be better for </body> and </html> to close them, allow trailing space, and then reparent
-                //  that space into body if other tags get re-added. but that's overkill for now
-                Element html = tb.popStackToClose("html");
                 tb.insert(t.asCharacter());
-                if (html != null) {
-                    tb.stack.add(html);
-                    Element body = html.selectFirst("body");
-                    if (body != null) tb.stack.add(body);
-                }
             }else if (t.isEOF()) {
                 // nice work chuck
             } else {
                 tb.error(this);
-                tb.transition(InBody);
+                tb.resetBody();
                 return tb.process(t);
             }
             return true;
@@ -1757,9 +1771,7 @@ enum HtmlTreeBuilderState {
         static final String[] Headings = new String[]{"h1", "h2", "h3", "h4", "h5", "h6"};
         static final String[] InBodyStartLiBreakers = new String[]{"address", "div", "p"};
         static final String[] DdDt = new String[]{"dd", "dt"};
-        static final String[] Formatters = new String[]{"b", "big", "code", "em", "font", "i", "s", "small", "strike", "strong", "tt", "u"};
         static final String[] InBodyStartApplets = new String[]{"applet", "marquee", "object"};
-        static final String[] InBodyStartEmptyFormatters = new String[]{"area", "br", "embed", "img", "keygen", "wbr"};
         static final String[] InBodyStartMedia = new String[]{"param", "source", "track"};
         static final String[] InBodyStartInputAttribs = new String[]{"action", "name", "prompt"};
         static final String[] InBodyStartDrop = new String[]{"caption", "col", "colgroup", "frame", "head", "tbody", "td", "tfoot", "th", "thead", "tr"};
