@@ -1,11 +1,21 @@
 package org.jsoup.select;
 
 import org.jsoup.helper.Validate;
+import org.jsoup.internal.StringUtil;
+import org.jsoup.nodes.Comment;
+import org.jsoup.nodes.DataNode;
 import org.jsoup.nodes.Element;
 import org.jsoup.nodes.FormElement;
 import org.jsoup.nodes.Node;
+import org.jsoup.nodes.TextNode;
 
-import java.util.*;
+import javax.annotation.Nullable;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collection;
+import java.util.HashSet;
+import java.util.LinkedHashSet;
+import java.util.List;
 
 /**
  A list of {@link Element}s, with methods that act on every element in the list.
@@ -65,7 +75,7 @@ public class Elements extends ArrayList<Element> {
     }
 
     /**
-     Checks if any of the matched elements have this attribute set.
+     Checks if any of the matched elements have this attribute defined.
      @param attributeKey attribute key
      @return true if any of the elements have the attribute; false if none do.
      */
@@ -77,6 +87,22 @@ public class Elements extends ArrayList<Element> {
         return false;
     }
 
+    /**
+     * Get the attribute value for each of the matched elements. If an element does not have this attribute, no value is
+     * included in the result set for that element.
+     * @param attributeKey the attribute name to return values for. You can add the {@code abs:} prefix to the key to
+     * get absolute URLs from relative URLs, e.g.: {@code doc.select("a").eachAttr("abs:href")} .
+     * @return a list of each element's attribute value for the attribute
+     */
+    public List<String> eachAttr(String attributeKey) {
+        List<String> attrs = new ArrayList<>(size());
+        for (Element element : this) {
+            if (element.hasAttr(attributeKey))
+                attrs.add(element.attr(attributeKey));
+        }
+        return attrs;
+    }
+
     /**
      * Set an attribute on all matched elements.
      * @param attributeKey attribute key
@@ -158,7 +184,8 @@ public class Elements extends ArrayList<Element> {
      */
     public String val() {
         if (size() > 0)
-            return first().val();
+            //noinspection ConstantConditions
+            return first().val(); // first() != null as size() > 0
         else
             return "";
     }
@@ -181,17 +208,23 @@ public class Elements extends ArrayList<Element> {
      * children, as the Element.text() method returns the combined text of a parent and all its children.
      * @return string of all text: unescaped and no HTML.
      * @see Element#text()
+     * @see #eachText()
      */
     public String text() {
-        StringBuilder sb = new StringBuilder();
+        StringBuilder sb = StringUtil.borrowBuilder();
         for (Element element : this) {
             if (sb.length() != 0)
                 sb.append(" ");
             sb.append(element.text());
         }
-        return sb.toString();
+        return StringUtil.releaseBuilder(sb);
     }
 
+    /**
+     Test if any matched Element has any text content, that is not just whitespace.
+     @return true if any element has non-blank text content.
+     @see Element#hasText()
+     */
     public boolean hasText() {
         for (Element element: this) {
             if (element.hasText())
@@ -199,6 +232,23 @@ public class Elements extends ArrayList<Element> {
         }
         return false;
     }
+
+    /**
+     * Get the text content of each of the matched elements. If an element has no text, then it is not included in the
+     * result.
+     * @return A list of each matched element's text content.
+     * @see Element#text()
+     * @see Element#hasText()
+     * @see #text()
+     */
+    public List<String> eachText() {
+        ArrayList<String> texts = new ArrayList<>(size());
+        for (Element el: this) {
+            if (el.hasText())
+                texts.add(el.text());
+        }
+        return texts;
+    }
     
     /**
      * Get the combined inner HTML of all matched elements.
@@ -207,13 +257,13 @@ public class Elements extends ArrayList<Element> {
      * @see #outerHtml()
      */
     public String html() {
-        StringBuilder sb = new StringBuilder();
+        StringBuilder sb = StringUtil.borrowBuilder();
         for (Element element : this) {
             if (sb.length() != 0)
                 sb.append("\n");
             sb.append(element.html());
         }
-        return sb.toString();
+        return StringUtil.releaseBuilder(sb);
     }
     
     /**
@@ -223,13 +273,13 @@ public class Elements extends ArrayList<Element> {
      * @see #html()
      */
     public String outerHtml() {
-        StringBuilder sb = new StringBuilder();
+        StringBuilder sb = StringUtil.borrowBuilder();
         for (Element element : this) {
             if (sb.length() != 0)
                 sb.append("\n");
             sb.append(element.outerHtml());
         }
-        return sb.toString();
+        return StringUtil.releaseBuilder(sb);
     }
 
     /**
@@ -244,8 +294,9 @@ public class Elements extends ArrayList<Element> {
     }
 
     /**
-     * Update the tag name of each matched element. For example, to change each {@code <i>} to a {@code <em>}, do
+     * Update (rename) the tag name of each matched element. For example, to change each {@code <i>} to a {@code <em>}, do
      * {@code doc.select("i").tagName("em");}
+     *
      * @param tagName the new tag name
      * @return this, for chaining
      * @see Element#tagName(String)
@@ -466,7 +517,7 @@ public class Elements extends ArrayList<Element> {
     }
 
     /**
-     * Get all of the following element siblings of each element in this list.
+     * Get each of the following element siblings of each element in this list.
      * @return all following element siblings.
      */
     public Elements nextAll() {
@@ -474,7 +525,7 @@ public class Elements extends ArrayList<Element> {
     }
 
     /**
-     * Get all of the following element siblings of each element in this list, filtered by the query.
+     * Get each of the following element siblings of each element in this list, that match the query.
      * @param query CSS query to match siblings against
      * @return all following element siblings.
      */
@@ -500,7 +551,7 @@ public class Elements extends ArrayList<Element> {
     }
 
     /**
-     * Get all of the previous element siblings of each element in this list.
+     * Get each of the previous element siblings of each element in this list.
      * @return all previous element siblings.
      */
     public Elements prevAll() {
@@ -508,7 +559,7 @@ public class Elements extends ArrayList<Element> {
     }
 
     /**
-     * Get all of the previous element siblings of each element in this list, filtered by the query.
+     * Get each of the previous element siblings of each element in this list, that match the query.
      * @param query CSS query to match siblings against
      * @return all previous element siblings.
      */
@@ -516,7 +567,7 @@ public class Elements extends ArrayList<Element> {
         return siblings(query, false, true);
     }
 
-    private Elements siblings(String query, boolean next, boolean all) {
+    private Elements siblings(@Nullable String query, boolean next, boolean all) {
         Elements els = new Elements();
         Evaluator eval = query != null? QueryParser.parse(query) : null;
         for (Element e : this) {
@@ -538,7 +589,7 @@ public class Elements extends ArrayList<Element> {
      * @return all of the parents and ancestor elements of the matched elements
      */
     public Elements parents() {
-        HashSet<Element> combo = new LinkedHashSet<Element>();
+        HashSet<Element> combo = new LinkedHashSet<>();
         for (Element e: this) {
             combo.addAll(e.parents());
         }
@@ -550,7 +601,7 @@ public class Elements extends ArrayList<Element> {
      Get the first matched element.
      @return The first matched element, or <code>null</code> if contents is empty.
      */
-    public Element first() {
+    public @Nullable Element first() {
         return isEmpty() ? null : get(0);
     }
 
@@ -558,7 +609,7 @@ public class Elements extends ArrayList<Element> {
      Get the last matched element.
      @return The last matched element, or <code>null</code> if contents is empty.
      */
-    public Element last() {
+    public @Nullable Element last() {
         return isEmpty() ? null : get(size() - 1);
     }
 
@@ -568,11 +619,17 @@ public class Elements extends ArrayList<Element> {
      * @return this, for chaining
      */
     public Elements traverse(NodeVisitor nodeVisitor) {
-        Validate.notNull(nodeVisitor);
-        NodeTraversor traversor = new NodeTraversor(nodeVisitor);
-        for (Element el: this) {
-            traversor.traverse(el);
-        }
+        NodeTraversor.traverse(nodeVisitor, this);
+        return this;
+    }
+
+    /**
+     * Perform a depth-first filtering on each of the selected elements.
+     * @param nodeFilter the filter callbacks to perform on each node
+     * @return this, for chaining
+     */
+    public Elements filter(NodeFilter nodeFilter) {
+        NodeTraversor.filter(nodeFilter, this);
         return this;
     }
 
@@ -582,11 +639,48 @@ public class Elements extends ArrayList<Element> {
      * no forms.
      */
     public List<FormElement> forms() {
-        ArrayList<FormElement> forms = new ArrayList<FormElement>();
+        ArrayList<FormElement> forms = new ArrayList<>();
         for (Element el: this)
             if (el instanceof FormElement)
                 forms.add((FormElement) el);
         return forms;
     }
 
+    /**
+     * Get {@link Comment} nodes that are direct child nodes of the selected elements.
+     * @return Comment nodes, or an empty list if none.
+     */
+    public List<Comment> comments() {
+        return childNodesOfType(Comment.class);
+    }
+
+    /**
+     * Get {@link TextNode} nodes that are direct child nodes of the selected elements.
+     * @return TextNode nodes, or an empty list if none.
+     */
+    public List<TextNode> textNodes() {
+        return childNodesOfType(TextNode.class);
+    }
+
+    /**
+     * Get {@link DataNode} nodes that are direct child nodes of the selected elements. DataNode nodes contain the
+     * content of tags such as {@code script}, {@code style} etc and are distinct from {@link TextNode}s.
+     * @return Comment nodes, or an empty list if none.
+     */
+    public List<DataNode> dataNodes() {
+        return childNodesOfType(DataNode.class);
+    }
+
+    private <T extends Node> List<T> childNodesOfType(Class<T> tClass) {
+        ArrayList<T> nodes = new ArrayList<>();
+        for (Element el: this) {
+            for (int i = 0; i < el.childNodeSize(); i++) {
+                Node node = el.childNode(i);
+                if (tClass.isInstance(node))
+                    nodes.add(tClass.cast(node));
+            }
+        }
+        return nodes;
+    }
+
 }
